<template>
    <v-container fluid fill-height style="margin-bottom: 5rem">
        <create-category-dialog @category-created="updateSelectCategory" @close="toggleCategoryDialog"
                                :display="categoryDialog"/>
        <Head title="Create Post"/>
            <v-layout align-center justify-center>
                <v-flex xs12 sm12 md8>
                    <v-card>
                        <v-toolbar flat>
                            <v-toolbar-title>
                                <v-icon left>mdi-post-outline</v-icon>
                                {{ cardTitle }}
                            </v-toolbar-title>
                            <v-divider class="mx-4" inset vertical></v-divider>
                            <v-spacer></v-spacer>
                        </v-toolbar>
                        <v-divider style="margin: 0;"/>
                        <v-card-text>
                            <v-form>
                                <v-list dense>
                                    <v-list-group v-model="imageSelector" no-action :value="2"
                                                  prepend-icon="mdi-file-image">
                                        <template v-slot:activator>
                                            <v-list-item-title class="primary--text">Image caption for this post
                                                <span v-if="form.media.id">
                                                (Selected: {{ form.media.title | trim(30) }})
                                            </span>
                                                <span v-else>(VIDE)</span>
                                            </v-list-item-title>
                                        </template>

                                        <v-list-group sub-group v-for="media in medias" :key="media.id">
                                            <template v-slot:activator>
                                                <v-list-item-content>
                                                    <v-list-item-title>
                                                        {{ media.title }}
                                                    </v-list-item-title>
                                                </v-list-item-content>
                                            </template>

                                            <v-list-item @click="imageSelectorSelect(file)"
                                                         v-for="(file, i) in media.menu" :key="i" link>
                                                <v-list-item-icon>
                                                    <v-icon small v-if="file.type === 'image'">
                                                        mdi-file-image
                                                    </v-icon>
                                                    <v-icon small v-else-if="file.type === 'video'">
                                                        mdi-video-camera
                                                    </v-icon>
                                                    <v-icon small v-else-if="file.type === 'document'">
                                                        mdi-file-document
                                                    </v-icon>
                                                    <v-icon small v-else-if="file.type === 'audio'">
                                                        mdi-volume-high
                                                    </v-icon>
                                                </v-list-item-icon>

                                                <v-list-item-title>{{ file.title | trim(40) }}</v-list-item-title>
                                            </v-list-item>
                                        </v-list-group>
                                    </v-list-group>
                                </v-list>
                                <v-list dense>
                                    <v-autocomplete v-model="form.category" :items="categories" label="Select category"
                                                    outlined dense item-value="id" item-text="name"
                                                    :error-messages="errors.category" no-data-text="No category available">
                                        <template v-slot:append-item>
                                            <div class="text-center">
                                                <v-divider/>
                                                <v-btn class="mt-3 mb-2" color="primary" small outlined
                                                       @click="toggleCategoryDialog">
                                                    <v-icon left small>mdi-plus</v-icon>
                                                    Create one
                                                </v-btn>
                                            </div>
                                        </template>
                                    </v-autocomplete>
                                </v-list>
                                <v-text-field autofocus label="Post title" outlined
                                              type="text" v-model="form.title" dense
                                              :error-messages="errors.title"></v-text-field>
                                <v-textarea label="Post description" outlined
                                            v-model="form.description" dense
                                            :error-messages="errors.description"></v-textarea>
                                <editor api-key="cezptlzsieov98ucntjqr1r9tv2fhxx6wog51pyuw8kmhf9q"
                                        :init="tinyInit" v-model="form.content"/>
                            </v-form>
                        </v-card-text>
                        <v-card-actions>
                            <v-spacer></v-spacer>
                            <v-btn dark color="primary" :loading="loading" @click="saveArticle">
                                {{ btnText }}
                                <v-icon right dark>mdi-content-save</v-icon>
                            </v-btn>
                        </v-card-actions>
                    </v-card>
                </v-flex>
            </v-layout>
        </v-container>
</template>

<script>
import DashboardLayout from "../../Components/Layouts/Dashboard.vue";
import Post from "./Post.vue";
import Editor from '@tinymce/tinymce-vue'
import CreateCategoryDialog from "./Category/CreateCategoryDialog.vue";

export default {
    name: "CreatePost",
    layout: [DashboardLayout, Post],
    components: {CreateCategoryDialog, Editor},
    data() {
        return {
            loading: false,
            imageSelector: false,
            categoryDialog: false,
            medias: [],
            categories: this.$page.props.categories,
            tags: this.$page.props.tags,
            form: {
                title: '',
                description: '',
                content: '',
                media: {},
                category: null,
                tags: []
            },
            errors: {
                title: [],
                description: [],
                content: [],
                media: [],
                category: [],
                tags: []
            },
            tinyInit: {
                height: 500,
                menubar: 'insert format edit tools view',
                a11y_advanced_options: true,
                plugins: [
                    'advlist autolink lists link image media charmap print preview anchor',
                    'searchreplace visualblocks code fullscreen emoticons',
                    'insertdatetime media table paste code help wordcount'
                ],
                toolbar:
                    'undo redo | fontsizeselect | styleselect | fontselect | bold italic backcolor textcolor | \
                    alignleft aligncenter alignright alignjustify | \
                    bullist numlist outdent indent | removeformat | link image | media | code emoticons | help',
                image_advtab: true,
                convert_urls: false,
                image_list: this.$route('media.list', {type: 'image'}),
                audio_template_callback: function (data) {
                    return '<audio controls>' + '\n<source src="' + data.source + '"' + (data.sourcemime ? ' type="' + data.sourcemime + '"' : '') + ' />\n' + (data.altsource ? '<source src="' + data.altsource + '"' + (data.altsourcemime ? ' type="' + data.altsourcemime + '"' : '') + ' />\n' : '') + '</audio>';
                },
                font_formats: "Andale Mono=andale mono,times; Arial=arial,helvetica,sans-serif; Arial Black=arial black,avant garde; " +
                    "Book Antiqua=book antiqua,palatino; Comic Sans MS=comic sans ms,sans-serif; Courier New=courier new,courier; " +
                    "Georgia=georgia,palatino; Helvetica=helvetica; Impact=impact,chicago; Oswald=oswald; Garamond=EB Garamond; Moonrocks=Rubik Moonrocks; Symbol=symbol; " +
                    "Tahoma=tahoma,arial,helvetica,sans-serif; Terminal=terminal,monaco; Times New Roman=times new roman,times; " +
                    "Trebuchet MS=trebuchet ms,geneva; Verdana=verdana,geneva; Webdings=webdings; Wingdings=wingdings,zapf dingbats",
                content_style: "@import url('https://fonts.googleapis.com/css2?family=Oswald&display=swap'); " +
                    "@import url('https://fonts.googleapis.com/css2?family=EB+Garamond&family=Rubik+Moonrocks&display=swap'); body { font-family: 'EB Garamond', serif; }",
            }

        }
    },
    watch: {
    },
    computed: {
        isEdit() {
            return this.$route().params.article
        },
        btnText() {
            return this.$route().params.article ? "Update" : "Create"
        },
        cardTitle() {
            return this.$route().params.article ? "Update a post" : "Create a new post"
        },
        mediaItems() {
            return this.medias.map((media) => {
                if (media.menu.length) {
                    return media.menu.map(file => {
                        return {
                            id: file.id,
                            text: `${media.title}/${file.title}`,
                            value: file,
                            disabled: false
                        }
                    })
                } else {
                    return [{
                        id: null,
                        text: `${media.title}/[vide]`,
                        value: 'Vide',
                        disabled: true
                    }]
                }
            }).reduce((a, b) => {
                return a.concat(b)
            }, [])
        }
    },
    methods: {
        toggleCategoryDialog() {
            this.categoryDialog = !this.categoryDialog
        },
        updateSelectCategory(data) {
            this.categories.push(data)
            this.form.category = data.id
        },
        imageSelectorSelect(data) {
            this.form.media = data
            this.imageSelector = false
        },
        validateData() {
            return new Promise((resolve, reject) => {
                resolve()
            })
        },
        fetchCategories() {
            return new Promise((resolve, reject) => {
                axios.get(this.$route('post.category.all'))
                    .then(res => res.data)
                    .then(data => {
                        this.categories = data
                        resolve(data)
                    })
                    .catch(reject)
            })
        },
        saveArticle() {
            if (this.isEdit) {
                this.validateData()
                    .then(() => {
                        axios.post(this.$route('post.update', {article: this.isEdit}), this.form)
                            .then(res => res.data)
                            .then(async () => {
                                await this.showSnack("Post updated successfully.")
                            })
                    })
                    .catch((err) => this.$toast.error(err.toString()))
            } else {
                this.validateData()
                    .then(() => {
                        axios.post(this.$route('post.store'), this.form)
                            .then(res => res.data)
                            .then(async () => {
                                await this.showSnack("Post created successfully.")
                            })
                            .catch((err) => this.showSnack({message: err.toString(), type: 'error'}))
                    })
                    .catch(console.log)
            }
        }
    },
    async mounted() {
        await this.fetchCategories()
        await axios.get(this.$route('media.list', {type: 'image'}))
            .then(res => res.data)
            .then(data => (this.medias = data))
        if (this.isEdit) {
            let data = this.$page.props.article
            this.form.title = data.title
            this.form.description = data.description
            this.form.category = data.category_id
            this.form.content = data.body
            this.form.media = data.media_id ? this.medias.map(mv => mv.menu).reduce((prev, next) => (prev.concat(next))).find(m => m.id === data.media_id) : {}
        }
    }
}
</script>

<style scoped>
.list-item {
    display: inline-block;
    margin-right: 10px;
}

.list-enter-active, .list-leave-active {
    transition: all 0.5s;
}

.list-enter, .list-leave-to /* .list-leave-active below version 2.1.8 */
{
    opacity: 0;
    transform: translateY(30px);
}
</style>
